/*
	main.c

	igorkov / 2015 / fsp@igorkov.org
 */
#include <string.h>
#include <math.h>
#include "lpc13xx.h"
#include "type.h"

#include "gio_lpc13xx.h"

#include "usb.h"
#include "usbdesc.h"
#include "usbrom.h"

#include "config.h"

#include "io.h"
#include "iap.h"
#include "event.h"
#include "leds.h"

#include "nrf24l01.h"
#include "dht.h"
#include "timer.h"

#include "spi.h"

//  :
#define FIRMWARE_VERSION_H 0x02
#define FIRMWARE_VERSION_L 0x01

#define     EN_TIMER32_1    (1<<10)
#define     EN_IOCON        (1<<16)
#define     EN_USBREG       (1<<14)

USB_DEV_INFO DeviceInfo;
HID_DEVICE_INFO HidDevInfo;
ROM ** rom = (ROM **)0x1fff1ff8;

uint8_t abDataIn[64];
uint8_t abData[64 - 8];
uint8_t bLen = 0;
uint8_t bProcessed = 0;
uint8_t bError = 0;

extern void SoftwareReset(void);

extern uint32_t SystemFrequency;

/*
 *  Get HID Input Report -> InReport
 */
void GetInReport(uint8_t src[], uint32_t length)
{
	io_data_t *pdata = (io_data_t*)src;

	memset(src, 0x00, length);		

	if (bProcessed == 1)
	{
		bProcessed = 0;
		
		pdata->dev2pc.bRetStatus = bError;
		pdata->dev2pc.bLen = bLen;
		memcpy(pdata->dev2pc.abData, abData, bLen);
		
		led_red(0);
	}
	else
	{
		pdata->dev2pc.bRetStatus = 0xFF;
		pdata->dev2pc.bLen = 0;
	}
}

/*
 *  Set HID Output Report <- OutReport
 */
void SetOutReport(uint8_t dst[], uint32_t length)
{
	led_red(1);
	if (length > 64)
	{
		bError = 0xFD;
		bProcessed = 1;
	}
	else if (bProcessed != 0)
	{
	}
	else
	{
		bProcessed = 2;
		memcpy(abDataIn, dst, length);
	}
}

#define MODE_USB      0x00
#define MODE_NATIVE   0x01
#define MODE_CALIBRATION   0x02

uint8_t mode = MODE_USB;

/*
//   :
void led_blink(void)
{
	static uint8_t led_state = 0;
	led_state = led_state ? 0 : 1;
	led_red(led_state);
	
	event_set(0, led_blink, 500);
}
*/

void frw_dispatch(int p)
{
	// Disable peripheral:
	(*rom)->pUSBD->connect(FALSE);      // USB Disconnect
	SysTick->CTRL  = 0;                 // Systick irq disable
	NVIC_DisableIRQ(ADC_IRQn);          // ADC irq disable
	NVIC_DisableIRQ(TIMER_32_0_IRQn);   // Timer0 irq disable
	NVIC_DisableIRQ(UART_IRQn);         // UART irq disable
	NVIC_DisableIRQ(USB_IRQn);          // USB irq disable
					
	if (p == 0)
	{
		int sect;
		
		sect = iap_getlastsector();
		if (sect > 1)
		{
	#if defined( RELEASE )
			//   RELEASE-    0 !
			//  !
			
			// erase firmware:
			iap_prepare(1, sect);
			iap_erase(1, sect);
	#else
			// erase firmware:
			iap_prepare(0, sect);
			iap_erase(0, sect);
	#endif					
		}
	}
	else if (p == 1)
	{
		int n;
		uint32_t *data = (uint32_t*)0x10000000;
		
		//      :
		*data = 0xF387E238; // Bootloader flag!
		
		//    :
		for (n=0; n<1000000; n++) { __NOP(); }
		
		SoftwareReset();
	}

	//    , ..   erase 
	// ,    .
	while (1);
}

void PIOINT2_IRQHandler(void)
{
  	if ( LPC_GPIO2->MIS & (0x1<<4) )	// pio 2.4
  	{
		LPC_GPIO2->IC |= (0x1<<4);
  	}
}

void PIOINT3_IRQHandler(void)
{
  	if ( LPC_GPIO3->MIS & (0x1<<3) )	// pio 3.3
  	{
		LPC_GPIO3->IC |= (0x1<<3);
  	}
}

void ce_set(void)
{
	gio_set1(3,2);
}

void ce_clr(void)
{
	gio_set0(3,2);
}

void NRF24L01_Receive(char *Buf, int _Buffer_Size) {
	NRF24L01_CE_HIGH;
	Delay_us(130);

	while ((NRF24L01_Get_Status() & _RX_DR) != _RX_DR);

	NRF24L01_CE_LOW;

	NRF24L01_Read_RX_Buf(Buf, _Buffer_Size);
	NRF24L01_Clear_Interrupts();
}


void NRF24L01_Send(char *Buf, int _Buffer_Size) {
	NRF24L01_Write_TX_Buf(Buf, _Buffer_Size);

	NRF24L01_RF_TX();

	while ((NRF24L01_Get_Status() & _TX_DS) != _TX_DS);

	NRF24L01_Clear_Interrupts();
}

#define _Buffer_Size    8   // 1..32
char Buf[_Buffer_Size] = "HOLA\0"; //Hello :)

extern uint32_t SystemFrequency; 

void read_dht(void)
{
	int ret;
	extern float humidity, temperature;
	
	//   :	
	NVIC_DisableIRQ(USB_IRQn);
	ret = dht_read();
	NVIC_EnableIRQ(USB_IRQn);
	
	if (ret == 0)
	{
		((float*)Buf)[0] = humidity;
		((float*)Buf)[1] = temperature;
		
		//   :
		NRF24L01_Send(Buf, _Buffer_Size);

		//delay_ms(5000);
		event_set(0, read_dht, 5000);
	} else {
		//delay_ms(1000)
		event_set(0, read_dht, 1000);
	}
}

int main(void)
{
	int ret;
	volatile int n;
	uint16_t crc;

	for (n = 0; n < 1000000; n++) { __NOP(); }

	// System Logic Init:
	SystemInit();

	// GPIO Initialize:
	gio_init(); 

	// LEDs init
	leds_init();

	// SPI init	
	spi_init();

	//  ,  1.
	//     :
	timer0_init(SystemFrequency/1000000);

	// DHT gpio init:
	dht_init();
		
	//   VBUS ,    :
	if (gio_get(0, 3))
	{
		//mode = MODE_USB;
		mode = MODE_NATIVE; // fix for sens, always NATIVE mode
	}
	else
	{
		mode = MODE_NATIVE;
	}

	if (mode == MODE_NATIVE)
	{
		#define	_CH             1   // Channel 0..125
		#define	_Address_Width  5   // 3..5
		char Address[_Address_Width] = { 0xAB, 0xDE, 0x54, 0xEF, 0x13 };

		// Event System Init:
		event_init();
		
		// Init RF-interface
		NRF24L01_Init(_TX_MODE, _CH, _1Mbps, Address, _Address_Width, _Buffer_Size);

		// 
		event_set(0, read_dht, 5000);
		
		// 
		while (1) {
			__WFI();
		}
	}
	else if (mode == MODE_USB)
	{
		//    .
		// NOP
	}
	else
	{
		//   .    .
		frw_dispatch(1);
	}

	//   USB-:
	HidDevInfo.idVendor = USB_VENDOR_ID;
	HidDevInfo.idProduct = USB_PROD_ID;
	HidDevInfo.bcdDevice = USB_DEVICE; 
	HidDevInfo.StrDescPtr = (uint32_t)&USB_StringDescriptor[0];
	HidDevInfo.InReportCount = 64;
	HidDevInfo.OutReportCount = 64;
	HidDevInfo.SampleInterval = 0x01;
	HidDevInfo.InReport = GetInReport;
	HidDevInfo.OutReport = SetOutReport;
	DeviceInfo.DevType = USB_DEVICE_CLASS_HUMAN_INTERFACE;
	DeviceInfo.DevDetailPtr = (uint32_t)&HidDevInfo;

	/* Enable Timer32_1, IOCON, and USB blocks (for USB ROM driver) */
	/* Already sets */
	//LPC_SYSCON->SYSAHBCLKCTRL |= (EN_TIMER32_1 | EN_IOCON | EN_USBREG);

#if 0
	/* Use pll and pin init function in rom */
	(*rom)->pUSBD->init_clk_pins();
#else
	// PLL already started
	LPC_IOCON->PIO0_3  &= ~0x07;              /* P0.3 VBUS */
	LPC_IOCON->PIO0_3  |=  0x01;              /* Select function USB_VBUS */
	LPC_IOCON->PIO0_6  &= ~0x07;              /* P0.6 SoftConnect */
	LPC_IOCON->PIO0_6  |=  0x01;              /* Selects function USB_CONNECT */
#endif

	/* insert a delay between clk init and usb init */
	for (n = 0; n < 75; n++) { __NOP(); }

	//  USB,   :
	(*rom)->pUSBD->init(&DeviceInfo); /* USB Initialization */
	(*rom)->pUSBD->connect(TRUE);     /* USB Connect */

	while (1)
	{
		if (bProcessed == 2)
		{
			io_data_t *pdata = (io_data_t *)abDataIn;
			switch (pdata->pc2dev.bOperation)
			{
#if 1
			case OP_GET_DHT22:
				NVIC_DisableIRQ(USB_IRQn);
				ret = dht_read();
				NVIC_EnableIRQ(USB_IRQn);
				if (ret == 0)
				{
					extern float humidity, temperature;
					((float*)abData)[0] = humidity;
					((float*)abData)[1] = temperature;
					bLen = sizeof(float)*2;
					bError = 0;
				}
				else
				{
					bLen = 0;
					bError = ret;
				}
				break;
#endif
			default:
				bError = 0xFE;
				bLen = 0;
				break;
			}
point_processed:
			bProcessed = 1;
		}
		__WFI();
	}
}

void USB_IRQHandler()
{
	(*rom)->pUSBD->isr();
}
